# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....claims.types.claim_status import ClaimStatus
from ....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from ....commons.types.charge_capture_id import ChargeCaptureId
from ....commons.types.charge_capture_post_billed_change_id import ChargeCapturePostBilledChangeId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.page_token import PageToken
from ....commons.types.pre_encounter_appointment_id import PreEncounterAppointmentId
from ....commons.types.pre_encounter_patient_id import PreEncounterPatientId
from ....commons.types.sort_direction import SortDirection
from ....encounters.resources.v_4.types.billable_status_type import BillableStatusType
from ....encounters.resources.v_4.types.responsible_party_type import ResponsiblePartyType
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.charge_capture import ChargeCapture
from .types.charge_capture_data import ChargeCaptureData
from .types.charge_capture_page import ChargeCapturePage
from .types.charge_capture_post_billed_change import ChargeCapturePostBilledChange
from .types.charge_capture_sort_field import ChargeCaptureSortField
from .types.charge_capture_status import ChargeCaptureStatus

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def create(
        self,
        *,
        data: ChargeCaptureData,
        charge_external_id: str,
        patient_external_id: str,
        status: ChargeCaptureStatus,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        data : ChargeCaptureData
            Charge Capture data contains all the fields needed to create an encounter, but listed as optional. Candid will use this data when attempting to bundle multiple Charge Captures into a single encounter.

        charge_external_id : str
            A client-specified unique ID to associate with this encounter; for example, your internal encounter ID or a Dr. Chrono encounter ID. This field should not contain PHI.

        patient_external_id : str
            The patient ID from the external EMR platform for the patient

        status : ChargeCaptureStatus
            the status of the charge capture

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page). Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.charge_capture.resources.v_1 import (
            ChargeCaptureData,
            ChargeCaptureStatus,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.create(
            data=ChargeCaptureData(),
            charge_external_id="charge_external_id",
            patient_external_id="patient_external_id",
            status=ChargeCaptureStatus.PLANNED,
        )
        """
        _response = self._raw_client.create(
            data=data,
            charge_external_id=charge_external_id,
            patient_external_id=patient_external_id,
            status=status,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            ehr_source_url=ehr_source_url,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    def create_from_pre_encounter_patient(
        self,
        *,
        data: ChargeCaptureData,
        charge_external_id: str,
        pre_encounter_patient_id: PreEncounterPatientId,
        pre_encounter_appointment_ids: typing.Sequence[PreEncounterAppointmentId],
        status: ChargeCaptureStatus,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Create a Charge Capture from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        At encounter creation time, information from the provided patient and appointment objects will be populated
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Note that these fields should not be populated in the ChargeCaptureData property of this endpoint, as they will be overwritten at encounter creation time.

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        data : ChargeCaptureData
            Charge Capture data contains all the fields needed to create an encounter, but listed as optional. Candid will use this data when attempting to bundle multiple Charge Captures into a single encounter.

        charge_external_id : str
            A client-specified unique ID to associate with this encounter; for example, your internal encounter ID or a Dr. Chrono encounter ID. This field should not contain PHI.

        pre_encounter_patient_id : PreEncounterPatientId

        pre_encounter_appointment_ids : typing.Sequence[PreEncounterAppointmentId]

        status : ChargeCaptureStatus
            the status of the charge capture

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page). Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.charge_capture.resources.v_1 import (
            ChargeCaptureData,
            ChargeCaptureStatus,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.create_from_pre_encounter_patient(
            data=ChargeCaptureData(),
            charge_external_id="charge_external_id",
            pre_encounter_patient_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            pre_encounter_appointment_ids=[
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            ],
            status=ChargeCaptureStatus.PLANNED,
        )
        """
        _response = self._raw_client.create_from_pre_encounter_patient(
            data=data,
            charge_external_id=charge_external_id,
            pre_encounter_patient_id=pre_encounter_patient_id,
            pre_encounter_appointment_ids=pre_encounter_appointment_ids,
            status=status,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            ehr_source_url=ehr_source_url,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    def update_post_billed_changes(
        self,
        *,
        charge_capture_change_ids: typing.Sequence[ChargeCapturePostBilledChangeId],
        resolved: bool,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ChargeCapturePostBilledChange]:
        """
        Parameters
        ----------
        charge_capture_change_ids : typing.Sequence[ChargeCapturePostBilledChangeId]
            A list of UUIDs corresponding to ChargeCapturePostBilledChanges.
            All of the charges sent will be marked as resolved

        resolved : bool
            Whether the change has been resolved. If true, the change will be marked as resolved.
            If false, the change will be marked as unresolved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ChargeCapturePostBilledChange]
            This list of updates will always return at most 1 update that is not resolved. The singular update will contain the difference between the updated charge and the created encounter.

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.update_post_billed_changes(
            charge_capture_change_ids=[
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            ],
            resolved=True,
        )
        """
        _response = self._raw_client.update_post_billed_changes(
            charge_capture_change_ids=charge_capture_change_ids, resolved=resolved, request_options=request_options
        )
        return _response.data

    def update(
        self,
        charge_capture_id: ChargeCaptureId,
        *,
        data: typing.Optional[ChargeCaptureData] = OMIT,
        charge_external_id: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        patient_external_id: typing.Optional[str] = OMIT,
        status: typing.Optional[ChargeCaptureStatus] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        charge_capture_id : ChargeCaptureId

        data : typing.Optional[ChargeCaptureData]

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page).
            Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status : typing.Optional[ChargeCaptureStatus]
            the status of the charge capture

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.update(
            charge_capture_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.update(
            charge_capture_id,
            data=data,
            charge_external_id=charge_external_id,
            ehr_source_url=ehr_source_url,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            patient_external_id=patient_external_id,
            status=status,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, charge_capture_id: ChargeCaptureId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        charge_capture_id : ChargeCaptureId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.get(
            charge_capture_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(charge_capture_id, request_options=request_options)
        return _response.data

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        exclude_charges_linked_to_claims: typing.Optional[bool] = None,
        patient_external_id_ranked_sort: typing.Optional[str] = None,
        status_ranked_sort: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id_ranked_sort: typing.Optional[str] = None,
        date_of_service_min_ranked_sort: typing.Optional[dt.date] = None,
        date_of_service_max_ranked_sort: typing.Optional[dt.date] = None,
        search_term: typing.Optional[str] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        claim_ids_ranked_sort: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids_ranked_sort: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name_ranked_sort: typing.Optional[str] = None,
        primary_payer_ids_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapturePage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge captures that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge captures with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge captures with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge captures with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge captures with one of the names in this list.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        exclude_charges_linked_to_claims : typing.Optional[bool]
            Whether to exclude charge captures which are part of a claim creation.

        patient_external_id_ranked_sort : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status_ranked_sort : typing.Optional[ChargeCaptureStatus]
            The charge capture status to show first

        charge_external_id_ranked_sort : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min_ranked_sort : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max_ranked_sort : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        search_term : typing.Optional[str]
            Filter by any of the following fields: charge_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or charge external id.

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        claim_ids_ranked_sort : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to show first. This will return all charge captures that have a resulting claim with one of the IDs in this list.

        claim_creation_ids_ranked_sort : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to show first.

        billing_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        service_facility_name_ranked_sort : typing.Optional[str]
            A string to show first. This will return all charge captures with this service facility name.

        primary_payer_ids_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to show first. This will return all charge captures with one of the primary payer IDs in this list.

        rendering_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        rendering_provider_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to show first. This will return all charge captures with one of the names in this list.

        supervising_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        supervising_provider_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to show first. This will return all charge captures with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to sort by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags. This will return all charge captures with one of the tags.

        primary_payer_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to sort by. This will return all charge captures with one of the names.

        patient_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to sort by. This will return all charge captures with one of the names.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapturePage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture.v_1.get_all()
        """
        _response = self._raw_client.get_all(
            limit=limit,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            patient_external_id=patient_external_id,
            status=status,
            charge_external_id=charge_external_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            claim_ids=claim_ids,
            claim_creation_ids=claim_creation_ids,
            billing_provider_npis=billing_provider_npis,
            service_facility_name=service_facility_name,
            primary_payer_ids=primary_payer_ids,
            rendering_provider_npis=rendering_provider_npis,
            rendering_provider_names=rendering_provider_names,
            supervising_provider_npis=supervising_provider_npis,
            supervising_provider_names=supervising_provider_names,
            claim_creation_category=claim_creation_category,
            tags=tags,
            primary_payer_names=primary_payer_names,
            patient_names=patient_names,
            exclude_charges_linked_to_claims=exclude_charges_linked_to_claims,
            patient_external_id_ranked_sort=patient_external_id_ranked_sort,
            status_ranked_sort=status_ranked_sort,
            charge_external_id_ranked_sort=charge_external_id_ranked_sort,
            date_of_service_min_ranked_sort=date_of_service_min_ranked_sort,
            date_of_service_max_ranked_sort=date_of_service_max_ranked_sort,
            search_term=search_term,
            billable_status=billable_status,
            responsible_party=responsible_party,
            claim_ids_ranked_sort=claim_ids_ranked_sort,
            claim_creation_ids_ranked_sort=claim_creation_ids_ranked_sort,
            billing_provider_npis_ranked_sort=billing_provider_npis_ranked_sort,
            service_facility_name_ranked_sort=service_facility_name_ranked_sort,
            primary_payer_ids_ranked_sort=primary_payer_ids_ranked_sort,
            rendering_provider_npis_ranked_sort=rendering_provider_npis_ranked_sort,
            rendering_provider_names_ranked_sort=rendering_provider_names_ranked_sort,
            supervising_provider_npis_ranked_sort=supervising_provider_npis_ranked_sort,
            supervising_provider_names_ranked_sort=supervising_provider_names_ranked_sort,
            claim_status=claim_status,
            claim_creation_category_ranked_sort=claim_creation_category_ranked_sort,
            tags_ranked_sort=tags_ranked_sort,
            primary_payer_names_ranked_sort=primary_payer_names_ranked_sort,
            patient_names_ranked_sort=patient_names_ranked_sort,
            request_options=request_options,
        )
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def create(
        self,
        *,
        data: ChargeCaptureData,
        charge_external_id: str,
        patient_external_id: str,
        status: ChargeCaptureStatus,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        data : ChargeCaptureData
            Charge Capture data contains all the fields needed to create an encounter, but listed as optional. Candid will use this data when attempting to bundle multiple Charge Captures into a single encounter.

        charge_external_id : str
            A client-specified unique ID to associate with this encounter; for example, your internal encounter ID or a Dr. Chrono encounter ID. This field should not contain PHI.

        patient_external_id : str
            The patient ID from the external EMR platform for the patient

        status : ChargeCaptureStatus
            the status of the charge capture

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page). Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.charge_capture.resources.v_1 import (
            ChargeCaptureData,
            ChargeCaptureStatus,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.create(
                data=ChargeCaptureData(),
                charge_external_id="charge_external_id",
                patient_external_id="patient_external_id",
                status=ChargeCaptureStatus.PLANNED,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            data=data,
            charge_external_id=charge_external_id,
            patient_external_id=patient_external_id,
            status=status,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            ehr_source_url=ehr_source_url,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    async def create_from_pre_encounter_patient(
        self,
        *,
        data: ChargeCaptureData,
        charge_external_id: str,
        pre_encounter_patient_id: PreEncounterPatientId,
        pre_encounter_appointment_ids: typing.Sequence[PreEncounterAppointmentId],
        status: ChargeCaptureStatus,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Create a Charge Capture from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        At encounter creation time, information from the provided patient and appointment objects will be populated
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Note that these fields should not be populated in the ChargeCaptureData property of this endpoint, as they will be overwritten at encounter creation time.

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        data : ChargeCaptureData
            Charge Capture data contains all the fields needed to create an encounter, but listed as optional. Candid will use this data when attempting to bundle multiple Charge Captures into a single encounter.

        charge_external_id : str
            A client-specified unique ID to associate with this encounter; for example, your internal encounter ID or a Dr. Chrono encounter ID. This field should not contain PHI.

        pre_encounter_patient_id : PreEncounterPatientId

        pre_encounter_appointment_ids : typing.Sequence[PreEncounterAppointmentId]

        status : ChargeCaptureStatus
            the status of the charge capture

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page). Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.charge_capture.resources.v_1 import (
            ChargeCaptureData,
            ChargeCaptureStatus,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.create_from_pre_encounter_patient(
                data=ChargeCaptureData(),
                charge_external_id="charge_external_id",
                pre_encounter_patient_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                pre_encounter_appointment_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ],
                status=ChargeCaptureStatus.PLANNED,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_pre_encounter_patient(
            data=data,
            charge_external_id=charge_external_id,
            pre_encounter_patient_id=pre_encounter_patient_id,
            pre_encounter_appointment_ids=pre_encounter_appointment_ids,
            status=status,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            ehr_source_url=ehr_source_url,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    async def update_post_billed_changes(
        self,
        *,
        charge_capture_change_ids: typing.Sequence[ChargeCapturePostBilledChangeId],
        resolved: bool,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ChargeCapturePostBilledChange]:
        """
        Parameters
        ----------
        charge_capture_change_ids : typing.Sequence[ChargeCapturePostBilledChangeId]
            A list of UUIDs corresponding to ChargeCapturePostBilledChanges.
            All of the charges sent will be marked as resolved

        resolved : bool
            Whether the change has been resolved. If true, the change will be marked as resolved.
            If false, the change will be marked as unresolved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ChargeCapturePostBilledChange]
            This list of updates will always return at most 1 update that is not resolved. The singular update will contain the difference between the updated charge and the created encounter.

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.update_post_billed_changes(
                charge_capture_change_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ],
                resolved=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_post_billed_changes(
            charge_capture_change_ids=charge_capture_change_ids, resolved=resolved, request_options=request_options
        )
        return _response.data

    async def update(
        self,
        charge_capture_id: ChargeCaptureId,
        *,
        data: typing.Optional[ChargeCaptureData] = OMIT,
        charge_external_id: typing.Optional[str] = OMIT,
        ehr_source_url: typing.Optional[str] = OMIT,
        originating_system: typing.Optional[str] = OMIT,
        claim_creation_category: typing.Optional[str] = OMIT,
        patient_external_id: typing.Optional[str] = OMIT,
        status: typing.Optional[ChargeCaptureStatus] = OMIT,
        attachment_external_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        charge_capture_id : ChargeCaptureId

        data : typing.Optional[ChargeCaptureData]

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        ehr_source_url : typing.Optional[str]
            External URL reference that links to Charge Capture details within the external system (e.g. the EHR visit page).
            Send full URL format for the external link (e.g. https://emr_charge_capture_url.com/123).

        originating_system : typing.Optional[str]
            An optional string field denoting the originating system of the charge.

        claim_creation_category : typing.Optional[str]
            An optional string field denoting the user defined category of the claim creation.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status : typing.Optional[ChargeCaptureStatus]
            the status of the charge capture

        attachment_external_document_ids : typing.Optional[typing.Sequence[str]]
            Provide external attachment IDs which have been uploaded to Candid. They will be associated with the Encounter at Encounter creation time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.update(
                charge_capture_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            charge_capture_id,
            data=data,
            charge_external_id=charge_external_id,
            ehr_source_url=ehr_source_url,
            originating_system=originating_system,
            claim_creation_category=claim_creation_category,
            patient_external_id=patient_external_id,
            status=status,
            attachment_external_document_ids=attachment_external_document_ids,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, charge_capture_id: ChargeCaptureId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargeCapture:
        """
        Parameters
        ----------
        charge_capture_id : ChargeCaptureId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapture

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.get(
                charge_capture_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(charge_capture_id, request_options=request_options)
        return _response.data

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        exclude_charges_linked_to_claims: typing.Optional[bool] = None,
        patient_external_id_ranked_sort: typing.Optional[str] = None,
        status_ranked_sort: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id_ranked_sort: typing.Optional[str] = None,
        date_of_service_min_ranked_sort: typing.Optional[dt.date] = None,
        date_of_service_max_ranked_sort: typing.Optional[dt.date] = None,
        search_term: typing.Optional[str] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        claim_ids_ranked_sort: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids_ranked_sort: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name_ranked_sort: typing.Optional[str] = None,
        primary_payer_ids_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names_ranked_sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCapturePage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge captures that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge captures with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge captures with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge captures with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge captures with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge captures with one of the names in this list.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        exclude_charges_linked_to_claims : typing.Optional[bool]
            Whether to exclude charge captures which are part of a claim creation.

        patient_external_id_ranked_sort : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        status_ranked_sort : typing.Optional[ChargeCaptureStatus]
            The charge capture status to show first

        charge_external_id_ranked_sort : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min_ranked_sort : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max_ranked_sort : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        search_term : typing.Optional[str]
            Filter by any of the following fields: charge_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or charge external id.

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        claim_ids_ranked_sort : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to show first. This will return all charge captures that have a resulting claim with one of the IDs in this list.

        claim_creation_ids_ranked_sort : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to show first.

        billing_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        service_facility_name_ranked_sort : typing.Optional[str]
            A string to show first. This will return all charge captures with this service facility name.

        primary_payer_ids_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to show first. This will return all charge captures with one of the primary payer IDs in this list.

        rendering_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        rendering_provider_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to show first. This will return all charge captures with one of the names in this list.

        supervising_provider_npis_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to show first. This will return all charge captures with one of the NPIs in this list.

        supervising_provider_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to show first. This will return all charge captures with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to sort by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags. This will return all charge captures with one of the tags.

        primary_payer_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to sort by. This will return all charge captures with one of the names.

        patient_names_ranked_sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to sort by. This will return all charge captures with one of the names.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCapturePage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture.v_1.get_all()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all(
            limit=limit,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            patient_external_id=patient_external_id,
            status=status,
            charge_external_id=charge_external_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            claim_ids=claim_ids,
            claim_creation_ids=claim_creation_ids,
            billing_provider_npis=billing_provider_npis,
            service_facility_name=service_facility_name,
            primary_payer_ids=primary_payer_ids,
            rendering_provider_npis=rendering_provider_npis,
            rendering_provider_names=rendering_provider_names,
            supervising_provider_npis=supervising_provider_npis,
            supervising_provider_names=supervising_provider_names,
            claim_creation_category=claim_creation_category,
            tags=tags,
            primary_payer_names=primary_payer_names,
            patient_names=patient_names,
            exclude_charges_linked_to_claims=exclude_charges_linked_to_claims,
            patient_external_id_ranked_sort=patient_external_id_ranked_sort,
            status_ranked_sort=status_ranked_sort,
            charge_external_id_ranked_sort=charge_external_id_ranked_sort,
            date_of_service_min_ranked_sort=date_of_service_min_ranked_sort,
            date_of_service_max_ranked_sort=date_of_service_max_ranked_sort,
            search_term=search_term,
            billable_status=billable_status,
            responsible_party=responsible_party,
            claim_ids_ranked_sort=claim_ids_ranked_sort,
            claim_creation_ids_ranked_sort=claim_creation_ids_ranked_sort,
            billing_provider_npis_ranked_sort=billing_provider_npis_ranked_sort,
            service_facility_name_ranked_sort=service_facility_name_ranked_sort,
            primary_payer_ids_ranked_sort=primary_payer_ids_ranked_sort,
            rendering_provider_npis_ranked_sort=rendering_provider_npis_ranked_sort,
            rendering_provider_names_ranked_sort=rendering_provider_names_ranked_sort,
            supervising_provider_npis_ranked_sort=supervising_provider_npis_ranked_sort,
            supervising_provider_names_ranked_sort=supervising_provider_names_ranked_sort,
            claim_status=claim_status,
            claim_creation_category_ranked_sort=claim_creation_category_ranked_sort,
            tags_ranked_sort=tags_ranked_sort,
            primary_payer_names_ranked_sort=primary_payer_names_ranked_sort,
            patient_names_ranked_sort=patient_names_ranked_sort,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.clinical_trial_id import ClinicalTrialId
from .....non_insurance_payers.resources.v_1.types.non_insurance_payer_id import NonInsurancePayerId
from .mutable_clinical_trial import MutableClinicalTrial


class ClinicalTrial(MutableClinicalTrial):
    clinical_trial_id: ClinicalTrialId
    non_insurance_payer_id: NonInsurancePayerId
    is_active: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .contract import Contract
    from .contract_base import ContractBase
    from .contract_create import ContractCreate
    from .contract_create_union import (
        ContractCreateUnion,
        ContractCreateUnion_Institutional,
        ContractCreateUnion_Professional,
    )
    from .contract_id import ContractId
    from .contract_service_facility import ContractServiceFacility
    from .contract_service_facility_base import ContractServiceFacilityBase
    from .contract_service_facility_id import ContractServiceFacilityId
    from .contract_type import ContractType
    from .contract_union import ContractUnion, ContractUnion_Institutional, ContractUnion_Professional
    from .contract_update import ContractUpdate
    from .contract_update_union import (
        ContractUpdateUnion,
        ContractUpdateUnion_Institutional,
        ContractUpdateUnion_Professional,
    )
    from .contract_with_providers_union import (
        ContractWithProvidersUnion,
        ContractWithProvidersUnion_Institutional,
        ContractWithProvidersUnion_Professional,
    )
    from .contracting_provider_id import ContractingProviderId
    from .contracts_page import ContractsPage
    from .institutional_contract import InstitutionalContract
    from .institutional_contract_create import InstitutionalContractCreate
    from .institutional_contract_update import InstitutionalContractUpdate
    from .professional_contract import ProfessionalContract
    from .professional_contract_create import ProfessionalContractCreate
    from .professional_contract_update import ProfessionalContractUpdate
    from .rendering_providerid import RenderingProviderid
_dynamic_imports: typing.Dict[str, str] = {
    "Contract": ".contract",
    "ContractBase": ".contract_base",
    "ContractCreate": ".contract_create",
    "ContractCreateUnion": ".contract_create_union",
    "ContractCreateUnion_Institutional": ".contract_create_union",
    "ContractCreateUnion_Professional": ".contract_create_union",
    "ContractId": ".contract_id",
    "ContractServiceFacility": ".contract_service_facility",
    "ContractServiceFacilityBase": ".contract_service_facility_base",
    "ContractServiceFacilityId": ".contract_service_facility_id",
    "ContractType": ".contract_type",
    "ContractUnion": ".contract_union",
    "ContractUnion_Institutional": ".contract_union",
    "ContractUnion_Professional": ".contract_union",
    "ContractUpdate": ".contract_update",
    "ContractUpdateUnion": ".contract_update_union",
    "ContractUpdateUnion_Institutional": ".contract_update_union",
    "ContractUpdateUnion_Professional": ".contract_update_union",
    "ContractWithProvidersUnion": ".contract_with_providers_union",
    "ContractWithProvidersUnion_Institutional": ".contract_with_providers_union",
    "ContractWithProvidersUnion_Professional": ".contract_with_providers_union",
    "ContractingProviderId": ".contracting_provider_id",
    "ContractsPage": ".contracts_page",
    "InstitutionalContract": ".institutional_contract",
    "InstitutionalContractCreate": ".institutional_contract_create",
    "InstitutionalContractUpdate": ".institutional_contract_update",
    "ProfessionalContract": ".professional_contract",
    "ProfessionalContractCreate": ".professional_contract_create",
    "ProfessionalContractUpdate": ".professional_contract_update",
    "RenderingProviderid": ".rendering_providerid",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Contract",
    "ContractBase",
    "ContractCreate",
    "ContractCreateUnion",
    "ContractCreateUnion_Institutional",
    "ContractCreateUnion_Professional",
    "ContractId",
    "ContractServiceFacility",
    "ContractServiceFacilityBase",
    "ContractServiceFacilityId",
    "ContractType",
    "ContractUnion",
    "ContractUnion_Institutional",
    "ContractUnion_Professional",
    "ContractUpdate",
    "ContractUpdateUnion",
    "ContractUpdateUnion_Institutional",
    "ContractUpdateUnion_Professional",
    "ContractWithProvidersUnion",
    "ContractWithProvidersUnion_Institutional",
    "ContractWithProvidersUnion_Professional",
    "ContractingProviderId",
    "ContractsPage",
    "InstitutionalContract",
    "InstitutionalContractCreate",
    "InstitutionalContractUpdate",
    "ProfessionalContract",
    "ProfessionalContractCreate",
    "ProfessionalContractUpdate",
    "RenderingProviderid",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .health_care_code_information import HealthCareCodeInformation
from .treatment_code import TreatmentCode
from .treatment_code_qualifier import TreatmentCodeQualifier


class TreatmentCodeInformation(HealthCareCodeInformation):
    """
    This corresponds to the TC code qualifier value.
    """

    treatment_code_qualifier: TreatmentCodeQualifier
    treatment_code: TreatmentCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .service_line_create_base_base import ServiceLineCreateBaseBase


class UniversalServiceLineCreate(ServiceLineCreateBaseBase):
    diagnosis_pointers: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Indices (zero-indexed) of all the diagnoses this service line references
    """

    revenue_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A 4 digit code that specifies facility department or type of service arrangement for institutional service line items (837i). This code is not required for professional claim billing (837p).
    """

    procedure_code: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class ContractType(enum.StrEnum):
    PROFESSIONAL = "professional"
    """
    Professional contract
    """

    INSTITUTIONAL = "institutional"
    """
    Institutional contract
    """

    _UNKNOWN = "__CONTRACTTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ContractType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        professional: typing.Callable[[], T_Result],
        institutional: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ContractType.PROFESSIONAL:
            return professional()
        if self is ContractType.INSTITUTIONAL:
            return institutional()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# nopycln: file
import datetime as dt
from collections import defaultdict
from typing import Any, Callable, Dict, List, Mapping, Tuple, Type, TypeVar, Union, cast

import pydantic

IS_PYDANTIC_V2 = pydantic.VERSION.startswith("2.")

if IS_PYDANTIC_V2:
    from pydantic.v1.datetime_parse import parse_date as parse_date
    from pydantic.v1.datetime_parse import parse_datetime as parse_datetime
    from pydantic.v1.fields import ModelField as ModelField
    from pydantic.v1.json import ENCODERS_BY_TYPE as encoders_by_type  # type: ignore[attr-defined]
    from pydantic.v1.typing import get_args as get_args
    from pydantic.v1.typing import get_origin as get_origin
    from pydantic.v1.typing import is_literal_type as is_literal_type
    from pydantic.v1.typing import is_union as is_union
else:
    from pydantic.datetime_parse import parse_date as parse_date  # type: ignore[no-redef]
    from pydantic.datetime_parse import parse_datetime as parse_datetime  # type: ignore[no-redef]
    from pydantic.fields import ModelField as ModelField  # type: ignore[attr-defined, no-redef]
    from pydantic.json import ENCODERS_BY_TYPE as encoders_by_type  # type: ignore[no-redef]
    from pydantic.typing import get_args as get_args  # type: ignore[no-redef]
    from pydantic.typing import get_origin as get_origin  # type: ignore[no-redef]
    from pydantic.typing import is_literal_type as is_literal_type  # type: ignore[no-redef]
    from pydantic.typing import is_union as is_union  # type: ignore[no-redef]

from .datetime_utils import serialize_datetime
from typing_extensions import TypeAlias

T = TypeVar("T")
Model = TypeVar("Model", bound=pydantic.BaseModel)


def parse_obj_as(type_: Type[T], object_: Any) -> T:
    if IS_PYDANTIC_V2:
        adapter = pydantic.TypeAdapter(type_)  # type: ignore[attr-defined]
        return adapter.validate_python(object_)
    return pydantic.parse_obj_as(type_, object_)


def to_jsonable_with_fallback(obj: Any, fallback_serializer: Callable[[Any], Any]) -> Any:
    if IS_PYDANTIC_V2:
        from pydantic_core import to_jsonable_python

        return to_jsonable_python(obj, fallback=fallback_serializer)
    return fallback_serializer(obj)


class UniversalBaseModel(pydantic.BaseModel):
    class Config:
        populate_by_name = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
        # Allow fields beginning with `model_` to be used in the model
        protected_namespaces = ()

    def json(self, **kwargs: Any) -> str:
        kwargs_with_defaults = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        if IS_PYDANTIC_V2:
            return super().model_dump_json(**kwargs_with_defaults)  # type: ignore[misc]
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: Any) -> Dict[str, Any]:
        """
        Override the default dict method to `exclude_unset` by default. This function patches
        `exclude_unset` to work include fields within non-None default values.
        """
        # Note: the logic here is multiplexed given the levers exposed in Pydantic V1 vs V2
        # Pydantic V1's .dict can be extremely slow, so we do not want to call it twice.
        #
        # We'd ideally do the same for Pydantic V2, but it shells out to a library to serialize models
        # that we have less control over, and this is less intrusive than custom serializers for now.
        if IS_PYDANTIC_V2:
            kwargs_with_defaults_exclude_unset = {
                **kwargs,
                "by_alias": True,
                "exclude_unset": True,
                "exclude_none": False,
            }
            kwargs_with_defaults_exclude_none = {
                **kwargs,
                "by_alias": True,
                "exclude_none": True,
                "exclude_unset": False,
            }
            return deep_union_pydantic_dicts(
                super().model_dump(**kwargs_with_defaults_exclude_unset),  # type: ignore[misc]
                super().model_dump(**kwargs_with_defaults_exclude_none),  # type: ignore[misc]
            )

        _fields_set = self.__fields_set__.copy()

        fields = _get_model_fields(self.__class__)
        for name, field in fields.items():
            if name not in _fields_set:
                default = _get_field_default(field)

                # If the default values are non-null act like they've been set
                # This effectively allows exclude_unset to work like exclude_none where
                # the latter passes through intentionally set none values.
                if default is not None or ("exclude_unset" in kwargs and not kwargs["exclude_unset"]):
                    _fields_set.add(name)

                    if default is not None:
                        self.__fields_set__.add(name)

        kwargs_with_defaults_exclude_unset_include_fields = {
            "by_alias": True,
            "exclude_unset": True,
            "include": _fields_set,
            **kwargs,
        }

        return super().dict(**kwargs_with_defaults_exclude_unset_include_fields)


def _union_list_of_pydantic_dicts(source: List[Any], destination: List[Any]) -> List[Any]:
    converted_list: List[Any] = []
    for i, item in enumerate(source):
        destination_value = destination[i]
        if isinstance(item, dict):
            converted_list.append(deep_union_pydantic_dicts(item, destination_value))
        elif isinstance(item, list):
            converted_list.append(_union_list_of_pydantic_dicts(item, destination_value))
        else:
            converted_list.append(item)
    return converted_list


def deep_union_pydantic_dicts(source: Dict[str, Any], destination: Dict[str, Any]) -> Dict[str, Any]:
    for key, value in source.items():
        node = destination.setdefault(key, {})
        if isinstance(value, dict):
            deep_union_pydantic_dicts(value, node)
        # Note: we do not do this same processing for sets given we do not have sets of models
        # and given the sets are unordered, the processing of the set and matching objects would
        # be non-trivial.
        elif isinstance(value, list):
            destination[key] = _union_list_of_pydantic_dicts(value, node)
        else:
            destination[key] = value

    return destination


if IS_PYDANTIC_V2:

    class V2RootModel(UniversalBaseModel, pydantic.RootModel):  # type: ignore[name-defined, type-arg]
        pass

    UniversalRootModel: TypeAlias = V2RootModel
else:
    UniversalRootModel: TypeAlias = UniversalBaseModel  # type: ignore[misc, no-redef]


def encode_by_type(o: Any) -> Any:
    encoders_by_class_tuples: Dict[Callable[[Any], Any], Tuple[Any, ...]] = defaultdict(tuple)
    for type_, encoder in encoders_by_type.items():
        encoders_by_class_tuples[encoder] += (type_,)

    if type(o) in encoders_by_type:
        return encoders_by_type[type(o)](o)
    for encoder, classes_tuple in encoders_by_class_tuples.items():
        if isinstance(o, classes_tuple):
            return encoder(o)


def update_forward_refs(model: Type["Model"], **localns: Any) -> None:
    if IS_PYDANTIC_V2:
        model.model_rebuild(raise_errors=False)  # type: ignore[attr-defined]
    else:
        model.update_forward_refs(**localns)


# Mirrors Pydantic's internal typing
AnyCallable = Callable[..., Any]


def universal_root_validator(
    pre: bool = False,
) -> Callable[[AnyCallable], AnyCallable]:
    def decorator(func: AnyCallable) -> AnyCallable:
        if IS_PYDANTIC_V2:
            # In Pydantic v2, for RootModel we always use "before" mode
            # The custom validators transform the input value before the model is created
            return cast(AnyCallable, pydantic.model_validator(mode="before")(func))  # type: ignore[attr-defined]
        return cast(AnyCallable, pydantic.root_validator(pre=pre)(func))  # type: ignore[call-overload]

    return decorator


def universal_field_validator(field_name: str, pre: bool = False) -> Callable[[AnyCallable], AnyCallable]:
    def decorator(func: AnyCallable) -> AnyCallable:
        if IS_PYDANTIC_V2:
            return cast(AnyCallable, pydantic.field_validator(field_name, mode="before" if pre else "after")(func))  # type: ignore[attr-defined]
        return cast(AnyCallable, pydantic.validator(field_name, pre=pre)(func))

    return decorator


PydanticField = Union[ModelField, pydantic.fields.FieldInfo]


def _get_model_fields(model: Type["Model"]) -> Mapping[str, PydanticField]:
    if IS_PYDANTIC_V2:
        return cast(Mapping[str, PydanticField], model.model_fields)  # type: ignore[attr-defined]
    return cast(Mapping[str, PydanticField], model.__fields__)


def _get_field_default(field: PydanticField) -> Any:
    try:
        value = field.get_default()  # type: ignore[union-attr]
    except:
        value = field.default
    if IS_PYDANTIC_V2:
        from pydantic_core import PydanticUndefined

        if value == PydanticUndefined:
            return None
        return value
    return value



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.date import Date
from ...commons.types.encounter_external_id import EncounterExternalId


class AllocationEarmarkType_DateOfService(UniversalBaseModel):
    value: Date
    type: typing.Literal["date_of_service"] = "date_of_service"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class AllocationEarmarkType_ExternalEncounterId(UniversalBaseModel):
    value: EncounterExternalId
    type: typing.Literal["external_encounter_id"] = "external_encounter_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


AllocationEarmarkType = typing_extensions.Annotated[
    typing.Union[AllocationEarmarkType_DateOfService, AllocationEarmarkType_ExternalEncounterId],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .raw_client import AsyncRawDefaultClient, RawDefaultClient
from .types.auth_get_token_response import AuthGetTokenResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DefaultClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDefaultClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDefaultClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDefaultClient
        """
        return self._raw_client

    def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthGetTokenResponse:
        """
        <Callout intent="info">
        Candid Health SDKs automatically handle authentication workflows after configuring them with the `client_id` and
        `client_secret`.
        </Callout>

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/v2/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The `/auth/v2/token` endpoint accepts both `Content-Type: application/json` and `Content-Type: application/x-www-form-urlencoded`. The request body should contain the `client_id` and `client_secret` as follows:

        ```json
        {
          "client_id": "YOUR_CLIENT_ID",
          "client_secret": "YOUR_CLIENT_SECRET"
        }
        ```
        or as URL-encoded form data:

        ```
        client_id=YOUR_CLIENT_ID&client_secret=YOUR_CLIENT_SECRET
        ```

        The bearer token is a signed [JWT](https://jwt.io/). The public key for the JWT can be found [here](https://candidhealth.auth0.com/pem) for any verification workflows.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        <Callout intent="warning">
        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between
        requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an `HTTP 429 Too Many Requests` error.
        </Callout>

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthGetTokenResponse

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.auth.default.get_token(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        """
        _response = self._raw_client.get_token(
            client_id=client_id, client_secret=client_secret, request_options=request_options
        )
        return _response.data


class AsyncDefaultClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDefaultClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDefaultClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDefaultClient
        """
        return self._raw_client

    async def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthGetTokenResponse:
        """
        <Callout intent="info">
        Candid Health SDKs automatically handle authentication workflows after configuring them with the `client_id` and
        `client_secret`.
        </Callout>

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/v2/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The `/auth/v2/token` endpoint accepts both `Content-Type: application/json` and `Content-Type: application/x-www-form-urlencoded`. The request body should contain the `client_id` and `client_secret` as follows:

        ```json
        {
          "client_id": "YOUR_CLIENT_ID",
          "client_secret": "YOUR_CLIENT_SECRET"
        }
        ```
        or as URL-encoded form data:

        ```
        client_id=YOUR_CLIENT_ID&client_secret=YOUR_CLIENT_SECRET
        ```

        The bearer token is a signed [JWT](https://jwt.io/). The public key for the JWT can be found [here](https://candidhealth.auth0.com/pem) for any verification workflows.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        <Callout intent="warning">
        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between
        requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an `HTTP 429 Too Many Requests` error.
        </Callout>

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthGetTokenResponse

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.auth.default.get_token(
                client_id="YOUR_CLIENT_ID",
                client_secret="YOUR_CLIENT_SECRET",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_token(
            client_id=client_id, client_secret=client_secret, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .phone_number_type import PhoneNumberType


class PhoneNumberOptional(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.commons import PhoneNumberOptional, PhoneNumberType

    PhoneNumberOptional(
        number="1234567890",
        type=PhoneNumberType.HOME,
    )
    """

    number: typing.Optional[str] = None
    type: typing.Optional[PhoneNumberType] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.email import Email
from .....commons.types.phone_number import PhoneNumber
from .guarantor_base import GuarantorBase
from .guarantor_id import GuarantorId


class Guarantor(GuarantorBase):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.commons import (
        PhoneNumber,
        PhoneNumberType,
        State,
        StreetAddressShortZip,
    )
    from candid.resources.guarantor.resources.v_1 import Guarantor

    Guarantor(
        guarantor_id=uuid.UUID(
            "8bbdbe63-58d3-4d40-98c9-40403c050977",
        ),
        phone_numbers=[
            PhoneNumber(
                number="1234567890",
                type=PhoneNumberType.HOME,
            )
        ],
        phone_consent=True,
        email="johndoe@joincandidhealth.com",
        email_consent=True,
        auto_charge_consent=True,
        first_name="John",
        last_name="Doe",
        external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
        date_of_birth=datetime.date.fromisoformat(
            "2000-01-01",
        ),
        address=StreetAddressShortZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
    )
    """

    guarantor_id: GuarantorId
    phone_numbers: typing.List[PhoneNumber]
    phone_consent: bool
    email: typing.Optional[Email] = None
    email_consent: bool
    auto_charge_consent: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .drug_identification import DrugIdentification
from .service_line_denial_reason import ServiceLineDenialReason
from .test_result import TestResult


class ServiceLineUpdateBase(UniversalBaseModel):
    edit_reason: typing.Optional[str] = None
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line, factoring in quantity. If procedure_code is updated and this is not, the system will attempt to set it based on chargemasters entries and the service line’s quantity. For example, if a single unit has an entry of 100 cents and 2 units were rendered, the charge_amount_cents will be set to 200, if there is no chargemaster entry, it will default to the amount set in this field.
    """

    drug_identification: typing.Optional[DrugIdentification] = None
    denial_reason: typing.Optional[ServiceLineDenialReason] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    837p Loop2300, SV105. This enum is not used or required in 837i claims. If your organization does not intend to submit claims with a different place of service at the service line level, this field should not be populated. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    units: typing.Optional[ServiceLineUnits] = None
    procedure_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    If `procedure_code` is updated, and `charge_amount_cents` is not, then `charge_amount_cents` will be set by the system.
    """

    quantity: typing.Optional[Decimal] = pydantic.Field(default=None)
    """
    String representation of a Decimal that can be parsed by most libraries.
    For professional claims, a ServiceLine quantity cannot contain more than one digit of precision
    (Example: 1.1 is valid, 1.11 is not). For institutional claims, a ServiceLine quantity cannot contain
    more than three decimal digits of precision.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on a 837-P and SV2-02, C003-07 on a 837-I form.
    """

    date_of_service: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    date_of_service must be defined on either the encounter or the service lines but not both.
    """

    end_date_of_service: typing.Optional[dt.date] = None
    test_results: typing.Optional[typing.List[TestResult]] = pydantic.Field(default=None)
    """
    Contains a list of test results. Test result types may map to MEA-02 on the 837-P (ex: Hemoglobin, Hematocrit).
    This is unused by 837-i and ignored for institutional service lines.
    No more than 5 MEA-02 test results may be submitted per service line.
    Updating test results utilizes PUT semantics, so the test results on the service line will be set to whatever inputs are provided.
    """

    has_epsdt_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500. If the value is true, the box will be populated with “Y”. Otherwise, the box will not be populated. This box is not used on an 837i.
    """

    has_family_planning_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Maps to NTE02 loop 2400 on the EDI 837.
    """

    prior_authorization_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Prior authorization number for this service line. Maps to the appropriate REF segment on Loop 2400 of the EDI 837p. This is not used for institutional claims (EDI 837i).
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An external identifier for this service line. This is not used in the EDI 837, but can be used to identify the service line in external systems.
    This field should not contain PHI. Must be unique on a given claim.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .type_of_bill_frequency_code import TypeOfBillFrequencyCode
from .type_of_care_code import TypeOfCareCode
from .type_of_facility_code import TypeOfFacilityCode


class TypeOfBillCompositeBase(UniversalBaseModel):
    type_of_facility: TypeOfFacilityCode = pydantic.Field()
    """
    The FL04 Institutional type of facility code for the bill.  The second digit of the composite code.
    """

    type_of_care: TypeOfCareCode = pydantic.Field()
    """
    The FL04 Institutional type of care code for the bill.  The third digit of the composite code.
    """

    frequency_code: TypeOfBillFrequencyCode = pydantic.Field()
    """
    The FL04 Institutional frequency code for the bill.  The fourth digit of the composite code.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .admitting_diagnosis_code_qualifier import AdmittingDiagnosisCodeQualifier
from .health_care_code_information import HealthCareCodeInformation
from .icd_code import IcdCode


class AdmittingDiagnosis(HealthCareCodeInformation):
    """
    This corresponds to BJ or ABJ code qualifier values.
    """

    admitting_diagnosis_code_qualifier: AdmittingDiagnosisCodeQualifier
    admitting_diagnosis_code: IcdCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class Ethnicity(enum.StrEnum):
    HISPANIC_OR_LATINO = "HISPANIC_OR_LATINO"
    NOT_HISPANIC_OR_LATINO = "NOT_HISPANIC_OR_LATINO"
    UNKNOWN = "UNKNOWN"
    REFUSED = "REFUSED"
    _UNKNOWN = "__ETHNICITY_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Ethnicity":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        hispanic_or_latino: typing.Callable[[], T_Result],
        not_hispanic_or_latino: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        refused: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Ethnicity.HISPANIC_OR_LATINO:
            return hispanic_or_latino()
        if self is Ethnicity.NOT_HISPANIC_OR_LATINO:
            return not_hispanic_or_latino()
        if self is Ethnicity.UNKNOWN:
            return unknown()
        if self is Ethnicity.REFUSED:
            return refused()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class AccountType(enum.StrEnum):
    PATIENT = "PATIENT"
    INSURANCE = "INSURANCE"
    THIRD_PARTY_PAYER = "THIRD_PARTY_PAYER"
    _UNKNOWN = "__ACCOUNTTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AccountType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        patient: typing.Callable[[], T_Result],
        insurance: typing.Callable[[], T_Result],
        third_party_payer: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AccountType.PATIENT:
            return patient()
        if self is AccountType.INSURANCE:
            return insurance()
        if self is AccountType.THIRD_PARTY_PAYER:
            return third_party_payer()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class NonInsurancePayerDescriptionUpdate_Remove(UniversalBaseModel):
    type: typing.Literal["remove"] = "remove"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class NonInsurancePayerDescriptionUpdate_Set(UniversalBaseModel):
    value: str
    type: typing.Literal["set"] = "set"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


NonInsurancePayerDescriptionUpdate = typing_extensions.Annotated[
    typing.Union[NonInsurancePayerDescriptionUpdate_Remove, NonInsurancePayerDescriptionUpdate_Set],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class RelatedCausesCode(enum.StrEnum):
    CAA = "AA"
    """
    Auto Accident
    """

    CAB = "AB"
    """
    Abuse
    """

    CAP = "AP"
    """
    Another Party Responsible
    """

    CEM = "EM"
    """
    Employment
    """

    COA = "OA"
    """
    Other Accident
    """

    CZZ = "ZZ"
    """
    Mutually Defined
    """

    _UNKNOWN = "__RELATEDCAUSESCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "RelatedCausesCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        caa: typing.Callable[[], T_Result],
        cab: typing.Callable[[], T_Result],
        cap: typing.Callable[[], T_Result],
        cem: typing.Callable[[], T_Result],
        coa: typing.Callable[[], T_Result],
        czz: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is RelatedCausesCode.CAA:
            return caa()
        if self is RelatedCausesCode.CAB:
            return cab()
        if self is RelatedCausesCode.CAP:
            return cap()
        if self is RelatedCausesCode.CEM:
            return cem()
        if self is RelatedCausesCode.COA:
            return coa()
        if self is RelatedCausesCode.CZZ:
            return czz()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.payer_id import PayerId
from .....common.types.payer_plan_group_id import PayerPlanGroupId
from .network_type import NetworkType


class PayerPlanGroupFields(UniversalBaseModel):
    payer_plan_group_id: PayerPlanGroupId
    payer_id: PayerId
    payer_name: str
    plan_type: NetworkType

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.report_transmission_code import ReportTransmissionCode
from .....commons.types.report_type_code import ReportTypeCode
from .....encounter_attachments.resources.v_1.types.attachment_id import AttachmentId
from .attachment_inclusion import AttachmentInclusion


class UpdateSupplementalInformationRequest(UniversalBaseModel):
    attachment_id: typing.Optional[AttachmentId] = None
    attachment_report_type_code: typing.Optional[ReportTypeCode] = None
    attachment_transmission_code: typing.Optional[ReportTransmissionCode] = None
    attachment_control_number: typing.Optional[str] = None
    attachment_inclusion: typing.Optional[AttachmentInclusion] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....diagnoses.types.diagnosis_id import DiagnosisId
from .service_line_update_base import ServiceLineUpdateBase


class UniversalServiceLineUpdate(ServiceLineUpdateBase):
    revenue_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A 4 digit code that specifies facility department or type of service arrangement for institutional service line items (837i). This code is not required for professional claim billing (837p).
    """

    diagnosis_id_zero: typing.Optional[DiagnosisId] = None
    diagnosis_id_one: typing.Optional[DiagnosisId] = None
    diagnosis_id_two: typing.Optional[DiagnosisId] = None
    diagnosis_id_three: typing.Optional[DiagnosisId] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .billable_status_type import BillableStatusType
    from .cash_pay_payer_error_message import CashPayPayerErrorMessage
    from .claim_supplemental_information import ClaimSupplementalInformation
    from .claim_supplemental_information_optional import ClaimSupplementalInformationOptional
    from .clinical_note import ClinicalNote
    from .clinical_note_category import ClinicalNoteCategory
    from .clinical_note_category_create import ClinicalNoteCategoryCreate
    from .clinical_note_category_create_optional import ClinicalNoteCategoryCreateOptional
    from .clinical_note_optional import ClinicalNoteOptional
    from .coding_attribution_type import CodingAttributionType
    from .encounter import Encounter
    from .encounter_base import EncounterBase
    from .encounter_create import EncounterCreate
    from .encounter_create_from_pre_encounter import EncounterCreateFromPreEncounter
    from .encounter_deep_optional import EncounterDeepOptional
    from .encounter_external_id_uniqueness_error_type import EncounterExternalIdUniquenessErrorType
    from .encounter_guarantor_missing_contact_info_error_type import EncounterGuarantorMissingContactInfoErrorType
    from .encounter_optional import EncounterOptional
    from .encounter_owner_of_next_action_type import EncounterOwnerOfNextActionType
    from .encounter_page import EncounterPage
    from .encounter_patient_control_number_uniqueness_error_type import EncounterPatientControlNumberUniquenessErrorType
    from .encounter_rendering_or_attending_provider_required_error import (
        EncounterRenderingOrAttendingProviderRequiredError,
    )
    from .encounter_sort_options import EncounterSortOptions
    from .encounter_submission_expectation import EncounterSubmissionExpectation
    from .encounter_submission_origin_type import EncounterSubmissionOriginType
    from .encounter_update import EncounterUpdate
    from .epsdt_referral import EpsdtReferral
    from .epsdt_referral_optional import EpsdtReferralOptional
    from .intake_follow_up import IntakeFollowUp
    from .intake_follow_up_id import IntakeFollowUpId
    from .intake_question import IntakeQuestion
    from .intake_question_id import IntakeQuestionId
    from .intake_question_optional import IntakeQuestionOptional
    from .intake_response_and_follow_ups import IntakeResponseAndFollowUps
    from .intervention import Intervention
    from .intervention_category import InterventionCategory
    from .intervention_optional import InterventionOptional
    from .invalid_tag_names_error_type import InvalidTagNamesErrorType
    from .key_does_not_exist_error import KeyDoesNotExistError
    from .lab import Lab
    from .lab_code_type import LabCodeType
    from .lab_optional import LabOptional
    from .medication import Medication
    from .medication_optional import MedicationOptional
    from .multiple_instances_for_schema_error import MultipleInstancesForSchemaError
    from .note_category import NoteCategory
    from .patient_history_category import PatientHistoryCategory
    from .patient_history_category_enum import PatientHistoryCategoryEnum
    from .patient_history_category_optional import PatientHistoryCategoryOptional
    from .payer_plan_group_payer_does_not_match_insurance_card_error import (
        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
    )
    from .prior_authorization_number import PriorAuthorizationNumber
    from .responsible_party_type import ResponsiblePartyType
    from .rx_cui import RxCui
    from .schema_does_not_exist_error import SchemaDoesNotExistError
    from .schema_instance_validation_error import (
        SchemaInstanceValidationError,
        SchemaInstanceValidationError_KeyDoesNotExist,
        SchemaInstanceValidationError_MultipleInstancesForSchema,
        SchemaInstanceValidationError_SchemaDoesNotExist,
        SchemaInstanceValidationError_SchemaUnauthorizedAccess,
        SchemaInstanceValidationError_ValueDoesNotMatchKeyType,
    )
    from .schema_instance_validation_failure import SchemaInstanceValidationFailure
    from .schema_unauthorized_access_error import SchemaUnauthorizedAccessError
    from .service_authorization_exception_code import ServiceAuthorizationExceptionCode
    from .synchronicity_type import SynchronicityType
    from .value_does_not_match_key_type_error import ValueDoesNotMatchKeyTypeError
    from .vitals import Vitals
    from .vitals_update import VitalsUpdate
_dynamic_imports: typing.Dict[str, str] = {
    "BillableStatusType": ".billable_status_type",
    "CashPayPayerErrorMessage": ".cash_pay_payer_error_message",
    "ClaimSupplementalInformation": ".claim_supplemental_information",
    "ClaimSupplementalInformationOptional": ".claim_supplemental_information_optional",
    "ClinicalNote": ".clinical_note",
    "ClinicalNoteCategory": ".clinical_note_category",
    "ClinicalNoteCategoryCreate": ".clinical_note_category_create",
    "ClinicalNoteCategoryCreateOptional": ".clinical_note_category_create_optional",
    "ClinicalNoteOptional": ".clinical_note_optional",
    "CodingAttributionType": ".coding_attribution_type",
    "Encounter": ".encounter",
    "EncounterBase": ".encounter_base",
    "EncounterCreate": ".encounter_create",
    "EncounterCreateFromPreEncounter": ".encounter_create_from_pre_encounter",
    "EncounterDeepOptional": ".encounter_deep_optional",
    "EncounterExternalIdUniquenessErrorType": ".encounter_external_id_uniqueness_error_type",
    "EncounterGuarantorMissingContactInfoErrorType": ".encounter_guarantor_missing_contact_info_error_type",
    "EncounterOptional": ".encounter_optional",
    "EncounterOwnerOfNextActionType": ".encounter_owner_of_next_action_type",
    "EncounterPage": ".encounter_page",
    "EncounterPatientControlNumberUniquenessErrorType": ".encounter_patient_control_number_uniqueness_error_type",
    "EncounterRenderingOrAttendingProviderRequiredError": ".encounter_rendering_or_attending_provider_required_error",
    "EncounterSortOptions": ".encounter_sort_options",
    "EncounterSubmissionExpectation": ".encounter_submission_expectation",
    "EncounterSubmissionOriginType": ".encounter_submission_origin_type",
    "EncounterUpdate": ".encounter_update",
    "EpsdtReferral": ".epsdt_referral",
    "EpsdtReferralOptional": ".epsdt_referral_optional",
    "IntakeFollowUp": ".intake_follow_up",
    "IntakeFollowUpId": ".intake_follow_up_id",
    "IntakeQuestion": ".intake_question",
    "IntakeQuestionId": ".intake_question_id",
    "IntakeQuestionOptional": ".intake_question_optional",
    "IntakeResponseAndFollowUps": ".intake_response_and_follow_ups",
    "Intervention": ".intervention",
    "InterventionCategory": ".intervention_category",
    "InterventionOptional": ".intervention_optional",
    "InvalidTagNamesErrorType": ".invalid_tag_names_error_type",
    "KeyDoesNotExistError": ".key_does_not_exist_error",
    "Lab": ".lab",
    "LabCodeType": ".lab_code_type",
    "LabOptional": ".lab_optional",
    "Medication": ".medication",
    "MedicationOptional": ".medication_optional",
    "MultipleInstancesForSchemaError": ".multiple_instances_for_schema_error",
    "NoteCategory": ".note_category",
    "PatientHistoryCategory": ".patient_history_category",
    "PatientHistoryCategoryEnum": ".patient_history_category_enum",
    "PatientHistoryCategoryOptional": ".patient_history_category_optional",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardError": ".payer_plan_group_payer_does_not_match_insurance_card_error",
    "PriorAuthorizationNumber": ".prior_authorization_number",
    "ResponsiblePartyType": ".responsible_party_type",
    "RxCui": ".rx_cui",
    "SchemaDoesNotExistError": ".schema_does_not_exist_error",
    "SchemaInstanceValidationError": ".schema_instance_validation_error",
    "SchemaInstanceValidationError_KeyDoesNotExist": ".schema_instance_validation_error",
    "SchemaInstanceValidationError_MultipleInstancesForSchema": ".schema_instance_validation_error",
    "SchemaInstanceValidationError_SchemaDoesNotExist": ".schema_instance_validation_error",
    "SchemaInstanceValidationError_SchemaUnauthorizedAccess": ".schema_instance_validation_error",
    "SchemaInstanceValidationError_ValueDoesNotMatchKeyType": ".schema_instance_validation_error",
    "SchemaInstanceValidationFailure": ".schema_instance_validation_failure",
    "SchemaUnauthorizedAccessError": ".schema_unauthorized_access_error",
    "ServiceAuthorizationExceptionCode": ".service_authorization_exception_code",
    "SynchronicityType": ".synchronicity_type",
    "ValueDoesNotMatchKeyTypeError": ".value_does_not_match_key_type_error",
    "Vitals": ".vitals",
    "VitalsUpdate": ".vitals_update",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BillableStatusType",
    "CashPayPayerErrorMessage",
    "ClaimSupplementalInformation",
    "ClaimSupplementalInformationOptional",
    "ClinicalNote",
    "ClinicalNoteCategory",
    "ClinicalNoteCategoryCreate",
    "ClinicalNoteCategoryCreateOptional",
    "ClinicalNoteOptional",
    "CodingAttributionType",
    "Encounter",
    "EncounterBase",
    "EncounterCreate",
    "EncounterCreateFromPreEncounter",
    "EncounterDeepOptional",
    "EncounterExternalIdUniquenessErrorType",
    "EncounterGuarantorMissingContactInfoErrorType",
    "EncounterOptional",
    "EncounterOwnerOfNextActionType",
    "EncounterPage",
    "EncounterPatientControlNumberUniquenessErrorType",
    "EncounterRenderingOrAttendingProviderRequiredError",
    "EncounterSortOptions",
    "EncounterSubmissionExpectation",
    "EncounterSubmissionOriginType",
    "EncounterUpdate",
    "EpsdtReferral",
    "EpsdtReferralOptional",
    "IntakeFollowUp",
    "IntakeFollowUpId",
    "IntakeQuestion",
    "IntakeQuestionId",
    "IntakeQuestionOptional",
    "IntakeResponseAndFollowUps",
    "Intervention",
    "InterventionCategory",
    "InterventionOptional",
    "InvalidTagNamesErrorType",
    "KeyDoesNotExistError",
    "Lab",
    "LabCodeType",
    "LabOptional",
    "Medication",
    "MedicationOptional",
    "MultipleInstancesForSchemaError",
    "NoteCategory",
    "PatientHistoryCategory",
    "PatientHistoryCategoryEnum",
    "PatientHistoryCategoryOptional",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardError",
    "PriorAuthorizationNumber",
    "ResponsiblePartyType",
    "RxCui",
    "SchemaDoesNotExistError",
    "SchemaInstanceValidationError",
    "SchemaInstanceValidationError_KeyDoesNotExist",
    "SchemaInstanceValidationError_MultipleInstancesForSchema",
    "SchemaInstanceValidationError_SchemaDoesNotExist",
    "SchemaInstanceValidationError_SchemaUnauthorizedAccess",
    "SchemaInstanceValidationError_ValueDoesNotMatchKeyType",
    "SchemaInstanceValidationFailure",
    "SchemaUnauthorizedAccessError",
    "ServiceAuthorizationExceptionCode",
    "SynchronicityType",
    "ValueDoesNotMatchKeyTypeError",
    "Vitals",
    "VitalsUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class PatientPaymentStatus(enum.StrEnum):
    PENDING = "PENDING"
    PAID = "paid"
    CANCELED = "CANCELED"
    VOIDED = "voided"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"
    STRIPE_SUCCEEDED = "succeeded"
    STRIPE_PENDING = "pending"
    STRIPE_FAILED = "failed"
    STRIPE_REQUIRES_ACTION = "requires_action"
    STRIPE_CANCELED = "canceled"
    _UNKNOWN = "__PATIENTPAYMENTSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PatientPaymentStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        paid: typing.Callable[[], T_Result],
        canceled: typing.Callable[[], T_Result],
        voided: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        stripe_succeeded: typing.Callable[[], T_Result],
        stripe_pending: typing.Callable[[], T_Result],
        stripe_failed: typing.Callable[[], T_Result],
        stripe_requires_action: typing.Callable[[], T_Result],
        stripe_canceled: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PatientPaymentStatus.PENDING:
            return pending()
        if self is PatientPaymentStatus.PAID:
            return paid()
        if self is PatientPaymentStatus.CANCELED:
            return canceled()
        if self is PatientPaymentStatus.VOIDED:
            return voided()
        if self is PatientPaymentStatus.FAILED:
            return failed()
        if self is PatientPaymentStatus.COMPLETED:
            return completed()
        if self is PatientPaymentStatus.STRIPE_SUCCEEDED:
            return stripe_succeeded()
        if self is PatientPaymentStatus.STRIPE_PENDING:
            return stripe_pending()
        if self is PatientPaymentStatus.STRIPE_FAILED:
            return stripe_failed()
        if self is PatientPaymentStatus.STRIPE_REQUIRES_ACTION:
            return stripe_requires_action()
        if self is PatientPaymentStatus.STRIPE_CANCELED:
            return stripe_canceled()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing
import uuid

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.date import Date
from .....commons.types.regions import Regions
from .....organization_providers.resources.v_2.types.organization_provider import OrganizationProvider
from .....payers.resources.v_3.types.payer import Payer
from ...v_2.types.authorized_signatory import AuthorizedSignatory
from ...v_2.types.contract_status import ContractStatus
from ...v_2.types.insurance_types import InsuranceTypes
from .contract_id import ContractId
from .contract_service_facility import ContractServiceFacility


class ContractWithProvidersUnion_Professional(UniversalBaseModel):
    type: typing.Literal["professional"] = "professional"
    rendering_provider_ids: typing.Set[uuid.UUID]
    contract_id: ContractId
    contracting_provider: OrganizationProvider
    provider_count: int
    payer: Payer
    effective_date: Date
    expiration_date: typing.Optional[Date] = None
    regions: Regions
    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatory] = None
    commercial_insurance_types: InsuranceTypes
    medicare_insurance_types: InsuranceTypes
    medicaid_insurance_types: InsuranceTypes

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ContractWithProvidersUnion_Institutional(UniversalBaseModel):
    type: typing.Literal["institutional"] = "institutional"
    contract_service_facilities: typing.List[ContractServiceFacility]
    contract_id: ContractId
    contracting_provider: OrganizationProvider
    provider_count: int
    payer: Payer
    effective_date: Date
    expiration_date: typing.Optional[Date] = None
    regions: Regions
    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatory] = None
    commercial_insurance_types: InsuranceTypes
    medicare_insurance_types: InsuranceTypes
    medicaid_insurance_types: InsuranceTypes

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ContractWithProvidersUnion = typing_extensions.Annotated[
    typing.Union[ContractWithProvidersUnion_Professional, ContractWithProvidersUnion_Institutional],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UserMetadata_MachineUserMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import UserMetadata_HumanUserMetadata

    UserMetadata_HumanUserMetadata(
        first_name="John",
        last_name="Doe",
        email="john.doe@example.com",
    )
    """

    type: typing.Literal["machine_user_metadata"] = "machine_user_metadata"
    name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class UserMetadata_HumanUserMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import UserMetadata_HumanUserMetadata

    UserMetadata_HumanUserMetadata(
        first_name="John",
        last_name="Doe",
        email="john.doe@example.com",
    )
    """

    type: typing.Literal["human_user_metadata"] = "human_user_metadata"
    first_name: str
    last_name: str
    email: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from candid.resources.users.resources.v_2 import UserMetadata_HumanUserMetadata

UserMetadata_HumanUserMetadata(
    first_name="John",
    last_name="Doe",
    email="john.doe@example.com",
)
"""
UserMetadata = typing_extensions.Annotated[
    typing.Union[UserMetadata_MachineUserMetadata, UserMetadata_HumanUserMetadata], pydantic.Field(discriminator="type")
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .eligibility_check_error import EligibilityCheckError
from .eligibility_request import EligibilityRequest
from .parsed_response import ParsedResponse
from .request_correction import RequestCorrection


class EligibilityCheck(UniversalBaseModel):
    batch_id: typing.Optional[str] = None
    errors: typing.Optional[typing.List[EligibilityCheckError]] = None
    request: typing.Optional[EligibilityRequest] = None
    response: typing.Any
    parsed_response: typing.Optional[ParsedResponse] = None
    request_corrections: typing.Optional[typing.List[RequestCorrection]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.encounter_id import EncounterId
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.health_care_code_information_get_all_response import HealthCareCodeInformationGetAllResponse
from .types.health_care_code_information_update import HealthCareCodeInformationUpdate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def update(
        self,
        encounter_id: EncounterId,
        *,
        request: HealthCareCodeInformationUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HealthCareCodeInformationGetAllResponse:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : HealthCareCodeInformationUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthCareCodeInformationGetAllResponse

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.health_care_code_information.resources.v_1 import (
            HealthCareCodeInformationUpdate,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.health_care_code_information.v_1.update(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=HealthCareCodeInformationUpdate(),
        )
        """
        _response = self._raw_client.update(encounter_id, request=request, request_options=request_options)
        return _response.data

    def get_all_for_encounter(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HealthCareCodeInformationGetAllResponse:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthCareCodeInformationGetAllResponse

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.health_care_code_information.v_1.get_all_for_encounter(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get_all_for_encounter(encounter_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def update(
        self,
        encounter_id: EncounterId,
        *,
        request: HealthCareCodeInformationUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HealthCareCodeInformationGetAllResponse:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : HealthCareCodeInformationUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthCareCodeInformationGetAllResponse

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.health_care_code_information.resources.v_1 import (
            HealthCareCodeInformationUpdate,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.health_care_code_information.v_1.update(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=HealthCareCodeInformationUpdate(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(encounter_id, request=request, request_options=request_options)
        return _response.data

    async def get_all_for_encounter(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HealthCareCodeInformationGetAllResponse:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthCareCodeInformationGetAllResponse

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.health_care_code_information.v_1.get_all_for_encounter(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all_for_encounter(encounter_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class TestResultType(enum.StrEnum):
    HEMATOCRIT = "HEMATOCRIT"
    HEMOGLOBIN = "HEMOGLOBIN"
    LDL = "LDL"
    VITAMIN_D = "VITAMIN_D"
    _UNKNOWN = "__TESTRESULTTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TestResultType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        hematocrit: typing.Callable[[], T_Result],
        hemoglobin: typing.Callable[[], T_Result],
        ldl: typing.Callable[[], T_Result],
        vitamin_d: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TestResultType.HEMATOCRIT:
            return hematocrit()
        if self is TestResultType.HEMOGLOBIN:
            return hemoglobin()
        if self is TestResultType.LDL:
            return ldl()
        if self is TestResultType.VITAMIN_D:
            return vitamin_d()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .copay_estimation_recommendation_payload import CopayEstimationRecommendationPayload
from .medicare_advantage_recommendation_payload import MedicareAdvantageRecommendationPayload
from .user_configured_prompts_recommendation_payload import UserConfiguredPromptsRecommendationPayload


class EligibilityRecommendationPayload_MedicareAdvantage(UniversalBaseModel):
    type: typing.Literal["MEDICARE_ADVANTAGE"] = "MEDICARE_ADVANTAGE"
    payload: MedicareAdvantageRecommendationPayload

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EligibilityRecommendationPayload_CoordinationOfBenefits(UniversalBaseModel):
    type: typing.Literal["COORDINATION_OF_BENEFITS"] = "COORDINATION_OF_BENEFITS"
    payload: typing.Any

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EligibilityRecommendationPayload_CopayEstimation(UniversalBaseModel):
    type: typing.Literal["COPAY_ESTIMATION"] = "COPAY_ESTIMATION"
    payload: CopayEstimationRecommendationPayload

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EligibilityRecommendationPayload_UserConfiguredPrompts(UniversalBaseModel):
    type: typing.Literal["USER_CONFIGURED_PROMPTS"] = "USER_CONFIGURED_PROMPTS"
    payload: UserConfiguredPromptsRecommendationPayload

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


EligibilityRecommendationPayload = typing_extensions.Annotated[
    typing.Union[
        EligibilityRecommendationPayload_MedicareAdvantage,
        EligibilityRecommendationPayload_CoordinationOfBenefits,
        EligibilityRecommendationPayload_CopayEstimation,
        EligibilityRecommendationPayload_UserConfiguredPrompts,
    ],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.billing_provider_commercial_license_type import BillingProviderCommercialLicenseType
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .billing_provider_secondary_identification import BillingProviderSecondaryIdentification
from .encounter_provider_base import EncounterProviderBase


class BillingProviderUpdate(EncounterProviderBase):
    """
    The billing provider is the provider or business entity submitting the claim.
    Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider.
    From a payer's perspective, this represents the person or entity being reimbursed.
    When a contract exists with the target payer, the billing provider should be the entity contracted with the payer.
    In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the
    tax ID (TIN) that the provider gave to the payer during contracting.
    In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID.
    Box 33 on the CMS-1500 claim or Form Locator 1 on a UB-04 claim form.
    """

    address: typing.Optional[StreetAddressLongZip] = None
    tax_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.
    """

    npi: typing.Optional[str] = None
    taxonomy_code: typing.Optional[str] = None
    provider_commercial_license_type: typing.Optional[BillingProviderCommercialLicenseType] = pydantic.Field(
        default=None
    )
    """
    837i Loop2010BB G2
    Provider Commercial Number
    """

    secondary_identification: typing.Optional[BillingProviderSecondaryIdentification] = pydantic.Field(default=None)
    """
    Only one of provider_commercial_license_type or secondary_identification may be provided
    837i Loop2010BB G2
    Secondary Identification
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .type_of_bill_frequency_code import TypeOfBillFrequencyCode
from .type_of_care_code import TypeOfCareCode
from .type_of_facility_code import TypeOfFacilityCode


class TypeOfBillCompositeUpdate_RawCode(UniversalBaseModel):
    value: str
    type: typing.Literal["raw_code"] = "raw_code"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class TypeOfBillCompositeUpdate_CompositeCodes(UniversalBaseModel):
    """
    A type of bill can either be seen as a compositio of structured parts, or as a literal string.
    This union represents the potential to express a type of bill either as one of those two shapes.
    Passing a string, this update will be seen as a complete 4 digit code, along with a padding code 0, that
    should match a valid composition.
    Passing as a structure, the update will be seen as three parts and will be joined together by the server
    to represent a valid composition for you.
    Which you use depends upon whether you are "passing through" coded information or making a decision about its
    structure explicitly.
    """

    type: typing.Literal["composite_codes"] = "composite_codes"
    type_of_facility: TypeOfFacilityCode
    type_of_care: TypeOfCareCode
    frequency_code: TypeOfBillFrequencyCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


TypeOfBillCompositeUpdate = typing_extensions.Annotated[
    typing.Union[TypeOfBillCompositeUpdate_RawCode, TypeOfBillCompositeUpdate_CompositeCodes],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .payer_ids import PayerIds
from .support_state import SupportState


class ClearinghousePayerInfo(UniversalBaseModel):
    payer_name: str = pydantic.Field()
    """
    The clearinghouse display name of the payer.
    """

    professional_payer_ids: PayerIds = pydantic.Field()
    """
    Payer IDs for professional claims
    """

    institutional_payer_ids: PayerIds = pydantic.Field()
    """
    Payer IDs for institutional claims
    """

    professional_claims_support: SupportState = pydantic.Field()
    """
    The support state for claims submission
    """

    institutional_claims_support: SupportState = pydantic.Field()
    """
    The support state for institutional claims submission
    """

    eligibility_support: SupportState = pydantic.Field()
    """
    The support state for eligibility
    """

    remittance_support: SupportState = pydantic.Field()
    """
    The support state for remittance
    """

    claim_attachment_support: SupportState = pydantic.Field()
    """
    The support state for claim attachment submission
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ClaimInvoiceItem,
        ClaimInvoiceItemInfo,
        Invoice,
        InvoiceDestination,
        InvoiceDestinationMetadata,
        InvoiceInfo,
        InvoiceItemAttributionCreate,
        InvoiceItemAttributionCreate_ClaimId,
        InvoiceItemAttributionCreate_ServiceLineId,
        InvoiceItemAttributionCreate_Unattributed,
        InvoiceItemCreate,
        InvoiceItemInfo,
        InvoiceSortField,
        InvoiceStatus,
        ServiceLineInvoiceItem,
        UnattributedInvoiceItem,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ClaimInvoiceItem": ".types",
    "ClaimInvoiceItemInfo": ".types",
    "Invoice": ".types",
    "InvoiceDestination": ".types",
    "InvoiceDestinationMetadata": ".types",
    "InvoiceInfo": ".types",
    "InvoiceItemAttributionCreate": ".types",
    "InvoiceItemAttributionCreate_ClaimId": ".types",
    "InvoiceItemAttributionCreate_ServiceLineId": ".types",
    "InvoiceItemAttributionCreate_Unattributed": ".types",
    "InvoiceItemCreate": ".types",
    "InvoiceItemInfo": ".types",
    "InvoiceSortField": ".types",
    "InvoiceStatus": ".types",
    "ServiceLineInvoiceItem": ".types",
    "UnattributedInvoiceItem": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ClaimInvoiceItem",
    "ClaimInvoiceItemInfo",
    "Invoice",
    "InvoiceDestination",
    "InvoiceDestinationMetadata",
    "InvoiceInfo",
    "InvoiceItemAttributionCreate",
    "InvoiceItemAttributionCreate_ClaimId",
    "InvoiceItemAttributionCreate_ServiceLineId",
    "InvoiceItemAttributionCreate_Unattributed",
    "InvoiceItemCreate",
    "InvoiceItemInfo",
    "InvoiceSortField",
    "InvoiceStatus",
    "ServiceLineInvoiceItem",
    "UnattributedInvoiceItem",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .universal_encounter_create import UniversalEncounterCreate
    from .universal_encounter_create_base import UniversalEncounterCreateBase
    from .universal_encounter_create_from_pre_encounter import UniversalEncounterCreateFromPreEncounter
    from .universal_encounter_create_from_pre_encounter_base import UniversalEncounterCreateFromPreEncounterBase
    from .universal_encounter_update import UniversalEncounterUpdate
    from .universal_encounter_update_base import UniversalEncounterUpdateBase
_dynamic_imports: typing.Dict[str, str] = {
    "UniversalEncounterCreate": ".universal_encounter_create",
    "UniversalEncounterCreateBase": ".universal_encounter_create_base",
    "UniversalEncounterCreateFromPreEncounter": ".universal_encounter_create_from_pre_encounter",
    "UniversalEncounterCreateFromPreEncounterBase": ".universal_encounter_create_from_pre_encounter_base",
    "UniversalEncounterUpdate": ".universal_encounter_update",
    "UniversalEncounterUpdateBase": ".universal_encounter_update_base",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "UniversalEncounterCreate",
    "UniversalEncounterCreateBase",
    "UniversalEncounterCreateFromPreEncounter",
    "UniversalEncounterCreateFromPreEncounterBase",
    "UniversalEncounterUpdate",
    "UniversalEncounterUpdateBase",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Carc,
        ClaimAdjustmentReasonCode,
        PatientDischargeStatusCode,
        PointOfOriginForAdmissionOrVisitCode,
        Rarc,
        RemittanceAdviceRemarkCode,
        TypeOfAdmissionOrVisitCode,
        TypeOfBillComposite,
        TypeOfBillCompositeBase,
        TypeOfBillCompositeUpdate,
        TypeOfBillCompositeUpdate_CompositeCodes,
        TypeOfBillCompositeUpdate_RawCode,
        TypeOfBillFrequencyCode,
        TypeOfCareCode,
        TypeOfFacilityCode,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Carc": ".types",
    "ClaimAdjustmentReasonCode": ".types",
    "PatientDischargeStatusCode": ".types",
    "PointOfOriginForAdmissionOrVisitCode": ".types",
    "Rarc": ".types",
    "RemittanceAdviceRemarkCode": ".types",
    "TypeOfAdmissionOrVisitCode": ".types",
    "TypeOfBillComposite": ".types",
    "TypeOfBillCompositeBase": ".types",
    "TypeOfBillCompositeUpdate": ".types",
    "TypeOfBillCompositeUpdate_CompositeCodes": ".types",
    "TypeOfBillCompositeUpdate_RawCode": ".types",
    "TypeOfBillFrequencyCode": ".types",
    "TypeOfCareCode": ".types",
    "TypeOfFacilityCode": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Carc",
    "ClaimAdjustmentReasonCode",
    "PatientDischargeStatusCode",
    "PointOfOriginForAdmissionOrVisitCode",
    "Rarc",
    "RemittanceAdviceRemarkCode",
    "TypeOfAdmissionOrVisitCode",
    "TypeOfBillComposite",
    "TypeOfBillCompositeBase",
    "TypeOfBillCompositeUpdate",
    "TypeOfBillCompositeUpdate_CompositeCodes",
    "TypeOfBillCompositeUpdate_RawCode",
    "TypeOfBillFrequencyCode",
    "TypeOfCareCode",
    "TypeOfFacilityCode",
]
